/*******************************************************************
*
*       Simulated Maximum Likelihood estimation of the
*       stochastic volatiliy model
*       version 1.2
*       February, 1996
*
*       Jon Danielsson (c)
*       Department of Economics
*       University of Iceland
*       101 Reykjavik
*       Iceland
*       e-mail: jond@hag.hi.is
*       ftp  hag.hi.is  
*       WWW  http://www.hag.hi.is/~jond
*       
*       please cite in published work and inform the author.
*
******************************************************************/


#include <iostream>
#include <iomanip>
#include <cmath>
#include <fstream>
#include <stdlib.h>
#include "optimize.hpp"

//enum {YES,NO};
static double (*snrfunc)(double *);
double *ppar;
double *simu,*siml;
int igrad;

opt::opt(int x) {
	int i;
	dim = x;
	par = new double [dim+1];
	se = new double [dim+1];
	for(i=1;i<=dim;i++)se[i] = 0.0;
	gradient=NO;
}
opt::~opt() { 
	delete [] par ;
	delete [] se  ;
}



void opt::optimize(void)
{
	ppar = new double [dim+1];
#ifdef SIMPLEX
	nrc_simplex(dim); 
#else
	err("invalid optimizer specified");
#endif
}

void    hess(int k, double *x, double **y,  double delta, double (*funk)(double par[]));


int opt::nrc_simplex(int ndim)
{

	int i,nfunc,j;
	double *x,*y,*lam,**p;
	x=new double[1,ndim];
	y=new double[1,ndim+1];
	p=matrix(1,ndim+1,1,ndim);
	for(i=1;i<=ndim;i++) y[i] = par[i] ;
	lam=new double[ndim+1];
	///double lam[ndim+1];
	for (i=1;i<=ndim;i++){
	  lam[i] = .1;
	}
	for (i=1;i<=ndim+1;i++) {
	  for (j=1;j<=ndim;j++){
			x[j]=p[i][j]=(i == (j+1) ? lam[j]+par[j] : par[j]);
		        //cout << x[j] << endl ;
	  }
		y[i]=func(x);
		//cout << y[i] << endl ;
	}
	Simplex(p,y,ndim,tol,func,&nfunc);
	for(i=1;i<=ndim;i++) par[i] = p[1][i] ;
	for (i=1;i<=ndim+1;i++) {
		for (j=1;j<=ndim;j++)
			x[j]=p[i][j]=(i == (j+1) ? 0.001+par[j] : par[j]);
		y[i]=func(x);
	}
	Simplex(p,y,ndim,tol/10.0,func,&nfunc);

	for(i=1;i<=ndim;i++) par[i] = p[1][i] ;

//	value = fret1;

//	free_matrix(p,1,4,1,3);
//	delete [] y;
//	delete [] x;
	for (j=1;j<=ndim;j++)
			y[j] = p[1][j] ;

	return (int) NULL;
}

#define DEBUG

// simplex seems to need this
int monit(double *fmin, double *fmax,double *sim,int *n,int *is,int *ncall)
{
	return 0;
}
// hessian calls this

int objfun(int *mode, int *n, double *xbb, double *objf, double *objgrd, int *nstate, int *iuser, double *user)
{
	int i;
    --user;
    --iuser;
    --objgrd;
    --xbb;
	double *grad = new double[*n +1+1];
	for(i=1;i<=*n;i++) ppar[i] = xbb[i];
	*objf = snrfunc(ppar);
	return 0;
} 
