/*******************************************************************
*
*       Simulated Maximum Likelihood estimation of the
*       stochastic volatiliy model
*       version 1.2
*       February, 1996
*
*       Jon Danielsson (c)
*       Department of Economics
*       University of Iceland
*       101 Reykjavik
*       Iceland
*       e-mail: jond@hag.hi.is
*       ftp  hag.hi.is  
*       WWW  http://www.hag.hi.is/~jond
*       
*       please cite in published work and inform the author.
*
******************************************************************/

#include <iostream>
#include <iomanip>
#include <cmath>
#include <fstream>
#include <cstdio>
#include <stdlib.h>





double **matrix(int r1, int r2, int c1, int c2)
{
	int i;
	double **m;
	r2 += 1;
	c2 += 1;
	/* allocate pointers to rows */
	m= (double **) calloc((r2+1), sizeof(double)) ;
	if (!m) printf("allocation failure 1 in dmatrix()");

	/* allocate rows and set pointers to them */
	for(i=0;i<=r2;i++) {
		m[i]= (double *) calloc((c2+1),sizeof(double));
		if (!m[i]) printf("allocation failure 2 in dmatrix()");
	}
	/* return pointer to array of pointers to rows */
	return m;
}
void free_matrix(double **m, int r1,int r2, int c1, int c2)
{
	int i;
	r2 += 1;
	c2 += 1;
	for(i=r2;i>=0;i--)  if(m[i]) free(m[i]);
	if(m) free(m);
}

//  Simplex optimization


#define NMAX 5000
#define GET_PSUM \
                         for (j=1;j<=ndim;j++){\
                         for (sum=0.0,i=1;i<=mpts;i++) sum += p[i][j];\
                         psum[j]=sum;}

#define SWAP(a,b) {swap=(a);(a)=(b);(b)=swap;}

double SimplexU(double **p, double y[], double psum[], int ndim,
             double (*funk)(double []), int ihi, double fac);
void Simplex(double **p, double y[], int ndim, double ftol,
       double (*funk)(double []), int *nfunk);


void Simplex(double **p, double y[], int ndim, double ftol,double (*funk)(double []), int *nfunk)
{
       int i,ihi,ilo,inhi,j,mpts;
       mpts=ndim+1;
       double rtol,sum,swap,ysave,ytry;

       double *psum=new double[ndim+2];
       *nfunk=0;
       GET_PSUM
       for (;;) {
            ilo=1;
            ihi = y[1]>y[2] ? (inhi=2,1) : (inhi=1,2);
            for (i=1;i<=mpts;i++) {
                if (y[i] <= y[ilo]) ilo=i;
                if (y[i] > y[ihi]){
                    inhi=ihi;
                    ihi=i;
                } else if (y[i] > y[inhi] && i != ihi) inhi=i;
            }
            rtol=2.0*fabs(y[ihi]-y[ilo])/(fabs(y[ihi])+fabs(y[ilo]));
            if (rtol < ftol) { 
                SWAP(y[1],y[ilo])
                for (i=1;i<=ndim;i++) SWAP(p[1][i],p[ilo][i])
                break;
            }
            if (*nfunk >= NMAX) {
	      std::cout << std::endl<<"IINMAX exceeded"<<std::endl;
	      exit(1);
	   }
            *nfunk += 2;
            ytry=SimplexU(p,y,psum,ndim,funk,ihi,-1.0);
            if (ytry <= y[ilo])
                ytry=SimplexU(p,y,psum,ndim,funk,ihi,2.0);
            else if (ytry >= y[inhi]) {
                ysave=y[ihi];
                ytry=SimplexU(p,y,psum,ndim,funk,ihi,0.5);
                if (ytry >= ysave) {
                    for (i=1;i<=mpts;i++) {
                           if (i != ilo) {
                               for (j=1;j<ndim;j++)
                                  p[i][i]=psum[j]=0.5*(p[i][j]+p[ilo][j]);
                               y[i]=(*funk)(psum);
			   }
		}	   
                       *nfunk += ndim; 
                        GET_PSUM 
		}
                }else --(*nfunk); 
	}
             delete [] psum;
}
double SimplexU(double **p, double y[], double psum[], int ndim,
             double (*funk)(double []), int ihi, double fac)
{
             int j;
             double fac1,fac2,ytry,*ptry;

             ptry=new double [ndim+2];
             fac1=(1.0-fac)/ndim;
             fac2=fac1-fac;
             for (j=1;j<=ndim;j++) ptry[j]=psum[j]*fac1-p[ihi][j]*fac2;
             ytry=(*funk)(ptry);
             if (ytry < y[ihi]){
                 y[ihi]=ytry;
                 for (j=1;j<=ndim;j++){
                     psum[j] += ptry[j]-p[ihi][j];
                     p[ihi][j]=ptry[j];
                 }
             }

             delete [] ptry;
             return ytry;
}


